package vishnu.cluster;

import java.util.*;

class Iterative
{
    Vector vector;
    int wordCount;
    
    Iterative(int vsize)
    {
	wordCount = vsize;
    }
    
    public void printVector(double[] a)
    {
        int i;
	
	for (i = 0; i < wordCount; i++)
    	    System.out.println("v[" + i + "] = " + a[i]);
    }


    void cluster_center (Cluster cluster, float [][] vectors, float[] centroid)
    {
        int i;
	
      	if (cluster.id == -1){
      	    cluster_center (cluster.child_1, vectors, centroid);
      	    cluster_center (cluster.child_2, vectors, centroid);
    	}else{
            float[] doubleArray = vectors[cluster.id];
      	    for (i = 0; i < doubleArray.length; i++)
		centroid[i] += doubleArray[i];
        }
    }
    
    void compute_cluster_centers (ClusterArray clusters, int num_of_clusters, float [][] vectors)
    {
	int i, j; double ssum;
	Cluster clust;
	

	/**** for each hierarchical cluster, compute centroid ****/
	for (i = 0; i < num_of_clusters; i++){
	    clust = (Cluster)clusters.contents.elementAt(i);
	    clust.centroid = new float[wordCount];
	    cluster_center (clust, vectors, clust.centroid);
	    
	    for (ssum = j = 0; j < wordCount; j++)
		ssum += clust.centroid[j] * clust.centroid[j];
	    
	    if(ssum>0)                // normalise
		for(ssum = 1.0/Math.sqrt(ssum), j = 0; j < wordCount; j++)
		    clust.centroid[j] *= ssum;
	}
    }
    
    
    void place_item (ClusterArray clusters, int num_of_clusters, float [][] vectors, int item_id,  Distance sim_func)
    {
        Cluster clust = (Cluster)clusters.contents.elementAt(0);
	double minDist = sim_func.get(clust.centroid,vectors[item_id]);	
	int x_minDist = 0;
	
	for (int i = 1; i < num_of_clusters; i++){
            clust = (Cluster)clusters.contents.elementAt(i);
      	    double dist = sim_func.get(clust.centroid,vectors[item_id]);
	    
            if (dist < minDist){
		x_minDist = i;
		minDist = dist;
	    }
    	}
	
	//System.out.println("in " + x_minDist); 
	clust = (Cluster)clusters.contents.elementAt(x_minDist);
        clust.iter_items.addElement(new Integer(item_id));
	clust.iter_items_num++;
    }
    
    
    // add all documents not yet added to the existing cluster hierarchy (which is based on the 
    // limit first documents. Hence there are vector.length - limit docs to add
    // consequently index runs from limit (start_index) to vector.length (end_index)
    void clustering (ClusterArray clusters, int[] num_of_clusters, float[][] vectors, int start_index, int end_index,Distance sim_func)
    {
        int i;

	System.out.println("Start: " + start_index + " Finish: " + end_index);
	
	if (start_index >= end_index)
	    return;
	
	compute_cluster_centers(clusters, num_of_clusters[0], vectors);
	
	for (i = start_index; i < end_index; i++){
	    System.out.println("Placing document " + i);
	    place_item(clusters, num_of_clusters[0], vectors, i, sim_func);
	}
    }
}
